﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Threading;

namespace BrowserFlow
{
    public partial class OutOfBrowserPage : UserControl
    {
        DispatcherTimer _dt = null;
        bool _animating = false;


        public OutOfBrowserPage()
        {
            InitializeComponent();
        }

        private void ComboBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (null != cb)
            {
                // Hide anything showing
                HideAll();

                if (cb.SelectedIndex == 0)
                {
                    // Image
                    img.Visibility = Visibility.Visible;
                }
                else if (cb.SelectedIndex == 1)
                {
                    // MediaElement
                    me.Visibility = Visibility.Visible;

                    // Stop and play
                    me.Stop();
                    me.Play();
                }
                else if (cb.SelectedIndex == 2)
                {
                    // Get Timer
                    if (null == _dt)
                    {
                        _dt = new DispatcherTimer();
                        _dt.Interval = new TimeSpan(0, 0, 0, 0, 17);

                        _dt.Tick += (object send, EventArgs args) =>
                        {
                            htmlBrush.Redraw();
                        };
                    }

                    _dt.Start();

                    // WebBrowser (change to brush)
                    brush.Visibility = Visibility.Visible;

                    // Navigate to the Ad
                    wb.Source = new Uri("PrintedDocument.pdf", UriKind.Relative);
                }
            }
        }

        #region Handlers
        private void HideAll()
        {
            // Stop timer
            if (null != _dt)
            {
                _dt.Stop();
            }

            foreach (var child in contentPanel.Children)
                child.Visibility = Visibility.Collapsed;
        }

        private void Right_Click(object sender, RoutedEventArgs e)
        {
            right.Begin();

            // Enable left
            rightButton.IsEnabled = false;
            leftButton.IsEnabled = true;
        }

        private void Left_Click(object sender, RoutedEventArgs e)
        {
            // Set animating flag
            _animating = true;

            left.Begin();

            // Enable right
            rightButton.IsEnabled = true;
            leftButton.IsEnabled = false;
        }

        private void right_Completed(object sender, EventArgs e)
        {
            _animating = false;
        }

        #endregion

        #region Mouse Enter/Leave

        private void brush_MouseEnter(object sender, MouseEventArgs e)
        {
            Dispatcher.BeginInvoke(() =>
                {
                    if ((cb.SelectedIndex == 2) && !_animating)
                    {
                        brush.Visibility = Visibility.Collapsed;
                        wb.Visibility = Visibility.Visible;
                    }
                });
        }

        private void LayoutRoot_MouseMove(object sender, MouseEventArgs e)
        {
            if (cb.SelectedIndex == 2)
            {
                wb.Visibility = Visibility.Collapsed;
                brush.Visibility = Visibility.Visible;
            }
        }
        #endregion
    }
}
