﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.IO;
using System.Windows.Threading;
using System.Windows.Interop;
using System.ComponentModel;
using System.Windows.Controls.Primitives;
using System.IO.IsolatedStorage;
using System.Threading;
using System.Runtime.InteropServices.Automation;

namespace TrustedApp
{
    public partial class OutOfBrowserPage : UserControl
    {
        private List<MediaFile> _playList;
        private int _nowPlaying;
        private int _upNext;
        FileStream _lastStream = null;
        AdHost _ad;
        private ManualResetEvent _mre = new ManualResetEvent(true);

        public OutOfBrowserPage()
        {
            InitializeComponent();
        }

        #region Excel Import and Export

        #region Import code
        private void ImportFromExcel()
        {
            dynamic sheet = null;
            dynamic excel = null;

            // Attach
            try
            {
                excel = AutomationFactory.GetObject("Excel.Application");
            }
            catch { }

            if (null != excel)
            {
                sheet = excel.ActiveSheet;
            }

            if (null != sheet)
            {
                // Get current sheet
                int idx = 2;
                List<MediaFile> newList = new List<MediaFile>();
                MediaFile mf;

                // Navigate the playlist
                while (idx > 0)
                {
                    // Creat new MediaFile
                    mf = new MediaFile();

                    // If ID is, must be done
                    if (string.IsNullOrEmpty(sheet.Cells[idx, 1].Value))
                    {
                        idx = -1;
                    }
                    else
                    {
                        // Read cells from Excel
                        mf.ID = GetValue(sheet.Cells[idx, 1].Value);
                        mf.Artist = GetValue(sheet.Cells[idx, 2].Value);
                        mf.Title = GetValue(sheet.Cells[idx, 3].Value);
                        mf.Album = GetValue(sheet.Cells[idx, 4].Value);
                        mf.Genre = GetValue(sheet.Cells[idx, 5].Value);
                        mf.AlbumArtPath = GetValue(sheet.Cells[idx, 6].Value);

                        // Add new item
                        newList.Add(mf);

                        // Next item
                        idx++;
                    }
                }

                // Set list
                Dispatcher.BeginInvoke(() =>
                {
                    ResetPlayList(newList);
                });
            }
        }

        private void Import_Click(object sender, RoutedEventArgs e)
        {
            BackgroundWorker bw = new BackgroundWorker();

            // Do Work
            bw.DoWork += (object send, DoWorkEventArgs args) =>
            {
                ImportFromExcel();
            };

            // Work done
            bw.RunWorkerCompleted += (object send, RunWorkerCompletedEventArgs args) =>
            {
                // Reset cursor
                this.Cursor = Cursors.Arrow;
            };

            // Set Wait Cursor
            this.Cursor = Cursors.Wait;

            // Do work
            bw.RunWorkerAsync();
        }
        #endregion

        delegate void SheetChangeHandler(dynamic sheet, dynamic range);
        delegate void WindowDeactiveHandler(dynamic workbook, dynamic window);

        private dynamic ExportToExcel()
        {
            // UE On:       Excel.Application
            // All:         http://msdn.microsoft.com/en-us/library/bb149081.aspx
            // Application: http://msdn.microsoft.com/en-us/library/bb255823.aspx

            dynamic excel = null;

            try
            {
                // Attach if one is running
                excel = AutomationFactory.GetObject("Excel.Application");
            }
            catch { }

            if (null == excel)
            {
                try
                {
                    // Try creating
                    excel = AutomationFactory.CreateObject("Excel.Application");
                }
                catch { }
            }

            if (null != excel)
            {
                // Wait
                _mre.Reset();

                // Show excel
                excel.visible = true;

                // Create workbook
                dynamic workbook = excel.workbooks;
                workbook.Add();

                // Get current sheet
                dynamic sheet = excel.ActiveSheet;
                dynamic cell = null;
                int idx = 2;

                // Set headers
                sheet.Cells[1, 1].Value = "File";
                sheet.Cells[1, 2].Value = "Artist";
                sheet.Cells[1, 3].Value = "Title";
                sheet.Cells[1, 4].Value = "Album";
                sheet.Cells[1, 5].Value = "Genre";
                sheet.Cells[1, 6].Value = "Album Art Path";

                // Navigate the playlist
                foreach (MediaFile mf in _playList)
                {
                    cell = sheet.Cells[idx, 1];       // Row, Column

                    cell.Value = mf.ID;
                    cell.ColumnWidth = 25;

                    cell = sheet.Cells[idx, 2];
                    cell.Value = mf.Artist;
                    cell.ColumnWidth = 12;

                    cell = sheet.Cells[idx, 3];
                    cell.Value = mf.Title;
                    cell.ColumnWidth = 20;

                    cell = sheet.Cells[idx, 4];
                    cell.Value = mf.Album;
                    cell.ColumnWidth = 20;

                    cell = sheet.Cells[idx, 5];
                    cell.Value = mf.Genre;
                    cell.ColumnWidth = 12;

                    cell = sheet.Cells[idx, 6];
                    cell.Value = mf.AlbumArtPath;
                    cell.ColumnWidth = 25;

                    idx++;
                }

                // Set turn on AutoFilter
                sheet.Range("A:F").AutoFilter();
            }

            // return
            return excel;
        }

        private void Export_Click(object sender, RoutedEventArgs e)
        {
            BackgroundWorker bw = new BackgroundWorker();

            // Do Work
            bw.DoWork += (object send, DoWorkEventArgs args) =>
            {
                // Export
                dynamic excel = ExportToExcel();

                // Set cursor state
                Dispatcher.BeginInvoke(() =>
                {
                    this.Cursor = Cursors.Arrow;
                });

                // Hook change event
                if (null != excel)
                {
                    excel.sheetChange += new SheetChangeHandler(OnSheetChange);
                    excel.windowDeactivate += new WindowDeactiveHandler(OnCloseExcel);

                    // Wait
                    _mre.WaitOne();

                    // When exited, cleanup
                    excel.sheetChange -= new SheetChangeHandler(OnSheetChange);
                    excel.windowDeactive -= new WindowDeactiveHandler(OnCloseExcel);
                }
            };

            // Work done
            bw.RunWorkerCompleted += (object send, RunWorkerCompletedEventArgs args) =>
            {
                // Hide
                toolbar.SetButtonEnableState(toolbar.ImportButton, false);
            };

            // Set Wait Cursor
            this.Cursor = Cursors.Wait;

            // Do work
            bw.RunWorkerAsync();
        }

        private void OnSheetChange(dynamic sheet, dynamic range)
        {
            Dispatcher.BeginInvoke(() =>
            {
                toolbar.SetButtonEnableState(toolbar.ImportButton, true);
            });
        }

        private void OnCloseExcel(dynamic sheet, dynamic range)
        {
            _mre.Set();
        }

        #endregion

        #region Toast Click
        private void ShowToast()
        {
            NotificationWindow nw = new NotificationWindow();

            NowPlaying playing = new NowPlaying();

            playing.DataContext = _playList[_nowPlaying];
            nw.Content = playing;
            nw.Width = 360;

            playing.MouseLeftButtonDown += delegate(object sender, MouseButtonEventArgs e)
            {
                Application.Current.MainWindow.WindowState = WindowState.Normal;
                nw.Close();
            };


            nw.Show(5000);
        }

        private void Toast_Click(object sender, RoutedEventArgs e)
        {
            ShowToast();
        }
        #endregion

        #region Set Height (Minimize)
        private bool _min = false;

        private void Minimize_Click(object sender, MouseButtonEventArgs e)
        {
            Window main = Application.Current.MainWindow;

            if (!_min)
            {
                main.Height = 40;
                rot.Angle = 0;
            }
            else
            {
                main.Height = 340;
                rot.Angle = 180;
            }

            _min = !_min;
        }
        #endregion

        #region PlayNext()
        private void PlayNext()
        {
            // Ensure AutoPlay
            me.AutoPlay = true;

            // Play next
            OpenAndPlay(_upNext);

            // Show Toast if minimized
            if (Application.Current.MainWindow.WindowState == WindowState.Minimized)
            {
                ShowToast();
            }
        }
        #endregion

        #region Helpers
        #region OpenAndPlay()
        private void OpenAndPlay(int index)
        {
            PlayItem(index);

            // Show ad every third song
            if ((_nowPlaying > 0) && ((_nowPlaying % 3) == 0))
            {
                _ad.ShowAd();
            }
        }
        #endregion

        #region Loaded Event
        private void UserControl_Loaded(object sender, RoutedEventArgs e)
        {
            // Load play list from the hard drive (reads from Music folder)
            LoadPlayList();

            // Add event handlers for Toolbar buttons (Shuffle, Import/Export to Excel)
            HookToolbarEventHandlers();

            // Create WebBrowser (Bug Workaround)
            _ad = new AdHost();
            adHost.Children.Add(_ad);
        }
        #endregion

        #region Other Methods
        private void HookToolbarEventHandlers()
        {
            // Setup Shuffle Button
            toolbar.ShuffleButton.Click += (object send, RoutedEventArgs args) =>
            {
                // Set autoplay
                me.AutoPlay = true;

                // Stop playback
                ResetPlayList(MediaFile.Shuffle(_playList));
            };

            toolbar.ExportButton.Click += new RoutedEventHandler(Export_Click);
            toolbar.ImportButton.Click += new RoutedEventHandler(Import_Click);
            toolbar.ToastButton.Click += new RoutedEventHandler(Toast_Click);
        }

        private void LoadPlayList()
        {
            // Setup MediaElement
            ac.Media = me;

            // Hook audio control events
            ac.MediaCompleted += new AudioControl.EventHandler(ac_MediaCompleted);

            // Get Shuffled Media Files
            _playList = MediaFile.Shuffle(MediaFile.GetMediaFiles());

            // Show
            playList.ItemsSource = _playList;

            // Play the first item
            OpenAndPlay(0);
        }

        private void PlayItem(int index)
        {
            // Check for open stream
            if (null != _lastStream)
            {
                me.Source = null;
                _lastStream.Close();
            }

            // Open and play
            _lastStream = File.Open(_playList[index].ID, FileMode.Open, FileAccess.Read);
            me.SetSource(_lastStream);

            // Setup now playing
            _nowPlaying = index;
            _upNext = (((_nowPlaying + 1) >= _playList.Count) ? 0 : (_nowPlaying + 1));

            // Setup TrackInfo
            nowPlaying.DataContext = _playList[_nowPlaying];
            upNext.DataContext = _playList[_upNext];

            // Set List
            if (playList.SelectedIndex != _nowPlaying)
            {
                playList.SelectedIndex = _nowPlaying;
                if (_nowPlaying > 0)
                {
                    playList.ScrollIntoView(playList.SelectedItem, playList.CurrentColumn);
                }
            }
        }

        void ResetPlayList(List<MediaFile> newList)
        {
            me.Stop();

            // Shuffle
            _playList = newList;

            // Reset
            playList.ItemsSource = _playList;
            OpenAndPlay(0);
        }

        void ac_MediaCompleted(object sender, EventArgs e)
        {
            // PlayNext
            PlayNext();
        }

        private void playList_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            DataGrid dg = (sender as DataGrid);

            // Play selected
            if (dg.SelectedIndex != _nowPlaying)
            {
                if (dg.SelectedIndex != 0)
                {
                    // Set AutoPlay
                    me.AutoPlay = true;
                }

                OpenAndPlay(dg.SelectedIndex);
            }
        }

        private string GetValue(dynamic value)
        {
            string result = null;

            if (null != value)
            {
                result = value.ToString();
            }

            return result;
        }
        #endregion
        #endregion
    }
}
